@@ -34,15 +34,13 @@ from twisted.python.filepath import FilePath
 from coherence.upnp.core.utils import parse_xml, ReverseProxyResource
 
 from coherence.upnp.core.DIDLLite import classChooser,Container,PhotoAlbum,Photo,ImageItem,Resource,DIDLElement
-from coherence.upnp.core.DIDLLite import simple_dlna_tags
+from coherence.upnp.core.DIDLLite import simple_dlna_tags, PlayContainerResource
 from coherence.upnp.core.soap_proxy import SOAPProxy
 from coherence.upnp.core.soap_service import errorCode
 
 from coherence.upnp.core.utils import getPage
 from coherence.backend import BackendItem, BackendStore
 
-import coherence.extern.louie as louie
-
 from coherence.extern.simple_plugin import Plugin
 
 from coherence import log
@@ -110,6 +108,7 @@ class FlickrItem(log.Loggable):
         self.id = id
         self.real_url = None
         self.obj = obj
+        self.upnp_class = UPnPClass
         self.store = store
         self.item = None
         self.date = None
@@ -189,15 +188,11 @@ class FlickrItem(log.Loggable):
                         obj.get('secret').encode('utf-8'))
 
         if parent == None:
-            parent_id = -1
+            self.parent_id = -1
         else:
-            parent_id = parent.get_id()
+            self.parent_id = parent.get_id()
 
         if self.mimetype == 'directory':
-            self.item = UPnPClass(self.id, parent_id, self.get_name())
-            if isinstance(self.item, Container):
-                self.item.childCount = 0
-            self.child_count = 0
             self.children = []
             self.update_id = 0
 
@@ -229,19 +224,12 @@ class FlickrItem(log.Loggable):
 
     def add_child(self, child, update=False):
         self.children.append(child)
-        self.child_count += 1
-        if isinstance(self.item, Container):
-            self.item.childCount += 1
         if update == True:
             self.update_id += 1
 
-
     def remove_child(self, child):
         self.info("remove_from %d (%s) child %d (%s)" % (self.id, self.get_name(), child.id, child.get_name()))
         if child in self.children:
-            self.child_count -= 1
-            if isinstance(self.item, Container):
-                self.item.childCount -= 1
             self.children.remove(child)
             self.update_id += 1
 
@@ -252,7 +240,7 @@ class FlickrItem(log.Loggable):
             return self.children[start:request_count]
 
     def get_child_count(self):
-        return self.child_count
+        return len(self.children)
 
     def get_id(self):
         return self.id
@@ -288,7 +276,14 @@ class FlickrItem(log.Loggable):
 
     def get_item(self):
         if self.item == None:
-            return self.create_item()
+            if self.mimetype == 'directory':
+                self.item = self.upnp_class(self.id, self.parent_id, self.get_name())
+                self.item.childCount = self.get_child_count()
+                if self.get_child_count() > 0:
+                    res = PlayContainerResource(self.store.server.uuid,cid=self.get_id(),fid=self.get_children()[0].get_id())
+                    self.item.res.append(res)
+            else:
+                return self.create_item()
         return self.item
 
     def create_item(self):
@@ -297,10 +292,16 @@ class FlickrItem(log.Loggable):
                 #print size.get('label'), size.get('source')
                 if size.get('label') == 'Original':
                     self.original_url = (size.get('source'),size.get('width')+'x'+size.get('height'))
-                    self.url = self.original_url[0]
+                    if self.store.proxy == False:
+                        self.url = self.original_url[0]
+                    else:
+                        self.location = ProxyImage(self.original_url[0])
                 elif size.get('label') == 'Large':
                     self.large_url = (size.get('source'),size.get('width')+'x'+size.get('height'))
-                    self.url = self.large_url[0]
+                    if self.store.proxy == False:
+                        self.url = self.large_url[0]
+                    else:
+                        self.location = ProxyImage(self.large_url[0])
                 elif size.get('label') == 'Medium':
                     self.medium_url = (size.get('source'),size.get('width')+'x'+size.get('height'))
                 elif size.get('label') == 'Small':
@@ -309,32 +310,54 @@ class FlickrItem(log.Loggable):
                     self.thumb_url = (size.get('source'),size.get('width')+'x'+size.get('height'))
 
             self.item = Photo(self.id,self.parent.get_id(),self.get_name())
+            #print self.id, self.store.proxy, self.url
             self.item.date = self.date
+            self.item.attachments = {}
             dlna_tags = simple_dlna_tags[:]
             dlna_tags[3] = 'DLNA.ORG_FLAGS=00f00000000000000000000000000000'
             if hasattr(self,'original_url'):
                 dlna_pn = 'DLNA.ORG_PN=JPEG_LRG'
-                res = Resource(self.original_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                if self.store.proxy == False:
+                    res = Resource(self.original_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                else:
+                    res = Resource(self.url+'?attachment=original', 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                    self.item.attachments['original'] = ProxyImage(self.original_url[0])
                 res.resolution = self.original_url[1]
                 self.item.res.append(res)
             elif hasattr(self,'large_url'):
                 dlna_pn = 'DLNA.ORG_PN=JPEG_LRG'
-                res = Resource(self.large_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                if self.store.proxy == False:
+                    res = Resource(self.large_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                else:
+                    res = Resource(self.url+'?attachment=large', 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                    self.item.attachments['large'] = ProxyImage(self.large_url[0])
                 res.resolution = self.large_url[1]
                 self.item.res.append(res)
             if hasattr(self,'medium_url'):
                 dlna_pn = 'DLNA.ORG_PN=JPEG_MED'
-                res = Resource(self.medium_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                if self.store.proxy == False:
+                    res = Resource(self.medium_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                else:
+                    res = Resource(self.url+'?attachment=medium', 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                    self.item.attachments['medium'] = ProxyImage(self.medium_url[0])
                 res.resolution = self.medium_url[1]
                 self.item.res.append(res)
             if hasattr(self,'small_url'):
                 dlna_pn = 'DLNA.ORG_PN=JPEG_SM'
-                res = Resource(self.small_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                if self.store.proxy == False:
+                    res = Resource(self.small_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                else:
+                    res = Resource(self.url+'?attachment=small', 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                    self.item.attachments['small'] = ProxyImage(self.small_url[0])
                 res.resolution = self.small_url[1]
                 self.item.res.append(res)
             if hasattr(self,'thumb_url'):
                 dlna_pn = 'DLNA.ORG_PN=JPEG_TN'
-                res = Resource(self.thumb_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                if self.store.proxy == False:
+                    res = Resource(self.thumb_url[0], 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                else:
+                    res = Resource(self.url+'?attachment=thumb', 'http-get:*:%s:%s' % (self.mimetype,';'.join([dlna_pn]+dlna_tags)))
+                    self.item.attachments['thumb'] = ProxyImage(self.thumb_url[0])
                 res.resolution = self.thumb_url[1]
                 self.item.res.append(res)
 
@@ -402,9 +425,10 @@ class FlickrStore(BackendStore):
            self.server.coherence.writeable_config() == True):
             if not None in (self.flickr_userid,self.flickr_password):
                 d = self.flickr_authenticate_app()
-                d.addBoth(lambda x: louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self))
-        else:
-            louie.send('Coherence.UPnP.Backend.init_completed', None, backend=self)
+                d.addBoth(lambda x: self.init_completed())
+                return
+            
+        self.init_completed()
 
     def __repr__(self):
         return str(self.__class__).split('.')[-1]
@@ -749,10 +773,14 @@ class FlickrStore(BackendStore):
         return d
 
     def flickr_photos_getSizes(self, photo_id=None):
-        d = self.flickr_call('flickr.photos.getSizes', photo_id=photo_id)
+        if self.flickr_authtoken != None:
+            api_sig = self.flickr_create_api_signature(auth_token=self.flickr_authtoken,method='flickr.photos.getSizes', photo_id=photo_id)
+            d = self.flickr_call('flickr.photos.getSizes', auth_token=self.flickr_authtoken, photo_id=photo_id,api_sig=api_sig)
+        else:
+            api_sig = self.flickr_create_api_signature(method='flickr.photos.getSizes', photo_id=photo_id)
+            d = self.flickr_call('flickr.photos.getSizes',photo_id=photo_id,api_sig=api_sig)
         return d
 
-
     def flickr_interestingness(self, date=None, per_page=100):
         if date == None:
             date = time.strftime( "%Y-%m-%d", time.localtime(time.time()-86400))
