@@ -21,6 +21,8 @@ mimetypes.add_type('audio/x-m4a', '.m4a')
 mimetypes.add_type('video/mp4', '.mp4')
 mimetypes.add_type('video/mpegts', '.ts')
 mimetypes.add_type('video/divx', '.divx')
+mimetypes.add_type('video/divx', '.avi')
+mimetypes.add_type('video/x-matroska', '.mkv')
 
 from urlparse import urlsplit
 
@@ -89,6 +91,7 @@ class FSItem(BackendItem):
         self.child_count = 0
         self.children = []
         self.sorted = False
+        self.caption = None
 
 
         if mimetype in ['directory','root']:
@@ -168,6 +171,7 @@ class FSItem(BackendItem):
                                      'application/ogg','audio/ogg',
                                      'audio/x-wav',
                                      'audio/x-m4a',
+                                     'audio/flac',
                                      'application/x-flac'):
                     dlna_pn = 'DLNA.ORG_PN=LPCM'
                     dlna_tags = simple_dlna_tags[:]
@@ -230,6 +234,21 @@ class FSItem(BackendItem):
                             self.item.attachments = {}
                         self.item.attachments[hash_from_path] = utils.StaticFile(urllib.quote(thumbnail))
 
+            if self.mimetype.startswith('video/'):
+                path = self.get_path()
+                caption,_ =  os.path.splitext(path)
+                caption = caption + '.srt'
+                if os.path.exists(caption):
+                    hash_from_path = str(id(caption))
+                    mimetype = 'smi/caption'
+                    new_res = Resource(self.url+'?attachment='+hash_from_path,
+                        'http-get:*:%s:%s' % (mimetype, '*'))
+                    new_res.size = os.path.getsize(caption)
+                    self.caption = new_res.data
+                    self.item.res.append(new_res)
+                    if not hasattr(self.item, 'attachments'):
+                        self.item.attachments = {}
+                    self.item.attachments[hash_from_path] = utils.StaticFile(urllib.quote(caption))
 
             try:
                 # FIXME: getmtime is deprecated in Twisted 2.6
@@ -403,6 +422,19 @@ class FSStore(BackendStore):
 
     implements = ['MediaServer']
 
+    description = """MediaServer exporting files from the file-system"""
+
+    options = [{'option':'name','type':'string','default':'my media','help': 'the name under this MediaServer shall show up with on other UPnP clients'},
+               {'option':'version','type':'int','default':2,'enum': (2,1),'help': 'the highest UPnP version this MediaServer shall support','level':'advance'},
+               {'option':'uuid','type':'string','help':'the unique (UPnP) identifier for this MediaServer, usually automatically set','level':'advance'},
+               {'option':'content','type':'string','default':xdg_content(),'help':'the path(s) this MediaServer shall export'},
+               {'option':'ignore_patterns','type':'string','help':'list of regex patterns, matching filenames will be ignored'},
+               {'option':'enable_inotify','type':'string','default':'yes','help':'enable real-time monitoring of the content folders'},
+               {'option':'enable_destroy','type':'string','default':'no','help':'enable deleting a file via an UPnP method'},
+               {'option':'import_folder','type':'string','help':'The path to store files imported via an UPnP method, if empty the Import method is disabled'}
+              ]
+
+
     def __init__(self, server, **kwargs):
         BackendStore.__init__(self,server,**kwargs)
         self.next_id = 1000
@@ -428,13 +460,16 @@ class FSStore(BackendStore):
 
         self.inotify = None
 
-        if haz_inotify == True:
-            try:
-                self.inotify = INotify()
-            except Exception,msg:
-                self.info("%s" %msg)
+        if kwargs.get('enable_inotify','yes') == 'yes':
+            if haz_inotify == True:
+                try:
+                    self.inotify = INotify()
+                except Exception,msg:
+                    self.info("%s" %msg)
+            else:
+                self.info("%s" %no_inotify_reason)
         else:
-            self.info("%s" %no_inotify_reason)
+            self.info("FSStore content auto-update disabled upon user request")
 
         if kwargs.get('enable_destroy','no') == 'yes':
             self.upnp_DestroyObject = self.hidden_upnp_DestroyObject
@@ -741,6 +776,8 @@ class FSStore(BackendStore):
                          'http-get:*:video/mpeg:*',
                          'internal:%s:video/avi:*' % self.server.coherence.hostname,
                          'http-get:*:video/avi:*',
+                         'internal:%s:video/divx:*' % self.server.coherence.hostname,
+                         'http-get:*:video/divx:*',
                          'internal:%s:video/quicktime:*' % self.server.coherence.hostname,
                          'http-get:*:video/quicktime:*',
                          'internal:%s:image/gif:*' % self.server.coherence.hostname,
