@@ -51,6 +51,18 @@ def is_audio(mimetype):
         return True
     return False
 
+def is_video(mimetype):
+    """ checks for type video,
+        expects a mimetype or an UPnP
+        protocolInfo
+    """
+    test = mimetype.split(':')
+    if len(test) == 4:
+        mimetype = test[2]
+    if mimetype.startswith('video/'):
+        return True
+    return False
+
 class Resources(list):
 
     """ a list of resources, always sorted after an append """
@@ -111,7 +123,7 @@ class Resources(list):
                    (remote_network == local_network or
                     remote_network == '*' or
                     local_network == '*') and
-                   (remote_content_format == local_content_format or
+                   (remote_content_format.startswith(local_content_format) or
                     remote_content_format == '*' or
                     local_content_format == '*')):
                         #print result, res
@@ -173,7 +185,7 @@ def build_dlna_additional_info(content_format,does_playcontainer=False):
         content_format = 'video/mpeg'
     if content_format in ['video/mp4','video/x-m4a']:
         additional_info = ['DLNA.ORG_PN=AVC_TS_BL_CIF15_AAC']+simple_dlna_tags
-    if content_format == 'video/x-msvideo':
+    if content_format in ['video/x-msvideo','video/avi','video/divx']:
         #additional_info = ';'.join(['DLNA.ORG_PN=MPEG4_P2_MP4_SP_AAC']+simple_dlna_tags)
         additional_info = ['*']
     if content_format == 'video/x-ms-wmv':
@@ -204,6 +216,7 @@ class Resource(object):
         self.size = None
         self.duration = None
 
+        self.nrAudioChannels = None
         self.resolution = None
 
         self.importUri = None
@@ -212,12 +225,14 @@ class Resource(object):
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if additional_info == '*':
                 self.protocolInfo = ':'.join((protocol,network,content_format,build_dlna_additional_info(content_format)))
+            elif additional_info == '#':
+                self.protocolInfo = ':'.join((protocol,network,content_format,'*'))
 
     def get_additional_info(self,upnp_client=''):
         protocol,network,content_format,additional_info = self.protocolInfo.split(':')
         if upnp_client  in ('XBox','Philips-TV',):
             """ we don't need the DLNA tags there,
-                and maybe they irritates these poor things anyway
+                and maybe they irritate these poor things anyway
             """
             additional_info = '*'
         elif upnp_client  in ('PLAYSTATION3',):
@@ -236,7 +251,7 @@ class Resource(object):
         root = ET.Element('res')
         if kwargs.get('upnp_client','') in ('XBox',):
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
-            if content_format == 'video/x-msvideo':
+            if content_format in ['video/divx','video/x-msvideo']:
                 content_format = 'video/avi'
             if content_format == 'audio/x-wav':
                 content_format = 'audio/wav'
@@ -245,7 +260,7 @@ class Resource(object):
         else:
             protocol,network,content_format,additional_info = self.protocolInfo.split(':')
             if content_format == 'video/x-msvideo':
-                content_format = 'video/avi'
+                content_format = 'video/divx'
             additional_info = self.get_additional_info(upnp_client=kwargs.get('upnp_client',''))
             root.attrib['protocolInfo'] = ':'.join((protocol,network,content_format,additional_info))
 
@@ -260,6 +275,9 @@ class Resource(object):
         if self.duration is not None:
             root.attrib['duration'] = self.duration
 
+        if self.nrAudioChannels is not None:
+            root.attrib['nrAudioChannels'] = self.nrAudioChannels
+
         if self.resolution is not None:
             root.attrib['resolution'] = self.resolution
 
@@ -302,6 +320,11 @@ class Resource(object):
         elif format == 'lpcm':
             dlna_pn = 'DLNA.ORG_PN=LPCM'
             content_format='audio/L16;rate=44100;channels=2'
+        elif format == 'mpegts':
+            if content_format == 'video/mpeg':
+                return None
+            dlna_pn = 'DLNA.ORG_PN=MPEG_PS_PAL' # 'DLNA.ORG_PN=MPEG_TS_SD_EU' # FIXME - don't forget HD
+            content_format='video/mpeg'
         else:
             return None
 
@@ -326,16 +349,12 @@ class PlayContainerResource(Resource):
                             sc='',md=0,
                             protocolInfo=None):
 
+        Resource.__init__(self)
         if cid == None:
             raise AttributeError('missing Container Id')
         if fid == None:
             raise AttributeError('missing first Child Id')
         self.protocolInfo = protocolInfo
-        self.bitrate = None
-        self.size = None
-        self.duration = None
-        self.resolution = None
-        self.importUri = None
 
         args = ['sid=' + urllib.quote(sid),
                 'cid=' + urllib.quote(str(cid)),
@@ -364,6 +383,8 @@ class Object(log.Loggable):
     date = None
     albumArtURI = None
     artist = None
+    genre = None
+    genres = None
     album = None
     originalTrackNumber=None
 
@@ -481,6 +502,13 @@ class Object(log.Loggable):
         if self.artist is not None:
             ET.SubElement(root, qname('artist',UPNP_NS)).text = self.artist
 
+        if self.genre is not None:
+            ET.SubElement(root, qname('genre',UPNP_NS)).text = self.genre
+
+        if self.genres is not None:
+            for genre in self.genres:
+                ET.SubElement(root, qname('genre',UPNP_NS)).text = genre
+
         if self.originalTrackNumber is not None:
             ET.SubElement(root, qname('originalTrackNumber',UPNP_NS)).text = str(self.originalTrackNumber)
 
@@ -528,6 +556,13 @@ class Object(log.Loggable):
                 self.longDescription = child.text
             elif child.tag.endswith('artist'):
                 self.artist = child.text
+            elif child.tag.endswith('genre'):
+                if self.genre != None:
+                    if self.genres == None:
+                        self.genres = [self.genre,]
+                    self.genres.append(child.text)
+                self.genre = child.text
+
             elif child.tag.endswith('album'):
                 self.album = child.text
             elif child.tag.endswith('class'):
@@ -556,6 +591,8 @@ class Item(Object):
     refID = None
 
     director = None
+    actors = None
+    language = None
 
     def __init__(self, *args, **kwargs):
         Object.__init__(self, *args, **kwargs)
@@ -570,6 +607,13 @@ class Item(Object):
         if self.refID is not None:
             ET.SubElement(root, 'refID').text = self.refID
 
+        if self.actors is not None:
+            for actor in self.actors:
+                ET.SubElement(root, qname('actor',DC_NS)).text = actor
+
+        #if self.language is not None:
+        #    ET.SubElement(root, qname('language',DC_NS)).text = self.language
+
         if kwargs.get('transcoding',False) == True:
             res = self.res.get_matching(['*:*:*:*'], protocol_type='http-get')
             if len(res) > 0 and is_audio(res[0].protocolInfo):
@@ -586,6 +630,13 @@ class Item(Object):
                     transcoded_res = old_res.transcoded('lpcm')
                     if transcoded_res != None:
                         root.append(transcoded_res.toElement(**kwargs))
+            elif len(res) > 0 and is_video(res[0].protocolInfo):
+                old_res = res[0]
+                for res in self.res:
+                    root.append(res.toElement(**kwargs))
+                transcoded_res = old_res.transcoded('mpegts')
+                if transcoded_res != None:
+                    root.append(transcoded_res.toElement(**kwargs))
             else:
                 for res in self.res:
                     root.append(res.toElement(**kwargs))
@@ -644,7 +695,6 @@ class AudioItem(Item):
 
     upnp_class = Item.upnp_class + '.audioItem'
 
-    genre = None
     publisher = None
     language = None
     relation = None
@@ -658,9 +708,6 @@ class AudioItem(Item):
 
         root = Item.toElement(self,**kwargs)
 
-        if self.genre is not None:
-            ET.SubElement(root, qname('genre',UPNP_NS)).text = self.genre
-
         if self.publisher is not None:
             ET.SubElement(root, qname('publisher',DC_NS)).text = self.publisher
 
@@ -720,10 +767,39 @@ class AudioBook(AudioItem):
 
 class VideoItem(Item):
     upnp_class = Item.upnp_class + '.videoItem'
+    valid_attrs = dict(genre=UPNP_NS, longDescription=UPNP_NS,
+                       producer=UPNP_NS, rating=UPNP_NS,
+                       actor=UPNP_NS, director=UPNP_NS,
+                       description=DC_NS, publisher=DC_NS, language=DC_NS,
+                       relation=DC_NS)
+
+    def toElement(self,**kwargs):
+        root = Item.toElement(self,**kwargs)
+
+        for attr_name, ns in self.valid_attrs.iteritems():
+            value = getattr(self, attr_name, None)
+            if value:
+                ET.SubElement(root, qname(attr_name, ns)).text = value
+
+        return root
+
+    def fromElement(self, elt):
+        Item.fromElement(self, elt)
+        for child in elt.getchildren():
+            tag = child.tag
+            val = child.text
+            if tag in self.valid_attrs.keys():
+                setattr(self, tag, val)
 
 class Movie(VideoItem):
     upnp_class = VideoItem.upnp_class + '.movie'
 
+    def __init__(self, *args, **kwargs):
+        VideoItem.__init__(self, *args, **kwargs)
+        self.valid_attrs.update(dict(storageMedium=UPNP_NS, DVDRegionCode=UPNP_NS,
+                                     channelName=UPNP_NS, scheduledStartTime=UPNP_NS,
+                                     sccheduledEndTime=UPNP_NS))
+
 class VideoBroadcast(VideoItem):
     upnp_class = VideoItem.upnp_class + '.videoBroadcast'
 
@@ -872,6 +948,14 @@ class DIDLElement(ElementInterface,log.Loggable):
                                    transcoding=self.transcoding))
         self._items.append(item)
 
+    def rebuild(self):
+        self._children = []
+        for item in self._items:
+            self.append(item.toElement(upnp_client=self.upnp_client,
+                                       parent_container=self.parent_container,
+                                       requested_id=self.requested_id,
+                                       transcoding=self.transcoding))
+
     def numItems(self):
         return len(self)
 
@@ -973,4 +1057,4 @@ if __name__ == '__main__':
     res.append(Resource('7','http-get:*:*:*'))
 
     for r in res:
-        print r.data, r.protocolInfo
\ No newline at end of file
+        print r.data, r.protocolInfo
