@@ -127,6 +127,11 @@ class Service(log.Loggable):
 
         return r
 
+    def as_dict(self):
+        d = {'type':self.service_type}
+        d['actions'] = [a.as_dict() for a in self._actions.values()]
+        return d
+
     def __repr__(self):
         return "Service %s %s" % (self.service_type,self.id)
 
@@ -200,6 +205,9 @@ class Service(log.Loggable):
     def get_actions(self):
         return self._actions
 
+    def get_scpdXML(self):
+        return self.scpdXML
+
     def get_action( self, name):
         try:
             return self._actions[name]
@@ -301,10 +309,10 @@ class Service(log.Loggable):
                                     #self.debug("%r update %r %r %r", self,namespace_uri, tag, var.attrib['val'])
                                     self.get_state_variable(tag, instance_id).update(var.attrib['val'])
                                     self.debug("updated 'attributed' var %r", var)
-                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value))
+                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value,event.raw))
             else:
                 self.get_state_variable(var_name, 0).update(var_value)
-                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value))
+                louie.send('Coherence.UPnP.DeviceClient.Service.Event.processed',None,self,(var_name,var_value,event.raw))
         if self.last_time_updated == None:
             # The clients (e.g. media_server_client) check for last time to detect whether service detection is complete
             # so we need to set it here and now to avoid a potential race condition
@@ -318,8 +326,8 @@ class Service(log.Loggable):
         def gotPage(x):
             #print "gotPage"
             #print x
-            data, headers = x
-            tree = utils.parse_xml(data, 'utf-8').getroot()
+            self.scpdXML, headers = x
+            tree = utils.parse_xml(self.scpdXML, 'utf-8').getroot()
             ns = "urn:schemas-upnp-org:service-1-0"
 
             for action_node in tree.findall('.//{%s}action' % ns):
@@ -664,6 +672,107 @@ class ServiceServer(log.Loggable):
         print "simulate_notification for", self.id
         self.set_variable(0, 'CurrentConnectionIDs', '0')
 
+    def get_scpdXML(self):
+        if not hasattr(self,'scpdXML') or self.scpdXML == None:
+            self.scpdXML = scpdXML(self)
+            self.scpdXML = self.scpdXML.build_xml()
+        return self.scpdXML
+
+    def register_vendor_variable(self,name,implementation='optional',
+                                      instance=0, evented='no',
+                                      data_type='string',
+                                      dependant_variable=None,
+                                      default_value=None,
+                                      allowed_values=None,has_vendor_values=False,allowed_value_range=None,
+                                      moderated=False):
+        """
+        enables a backend to add an own, vendor defined, StateVariable to the service
+
+        @ivar name: the name of the new StateVariable
+        @ivar implementation: either 'optional' or 'required'
+        @ivar instance: the instance number of the service that variable should be assigned to, usually '0'
+        @ivar evented: boolean, or the special keyword 'never' if the variable doesn't show up in a LastChange event too
+        @ivar data_type: 'string','boolean','bin.base64' or various number formats
+        @ivar dependant_variable: the name of another StateVariable that depends on this one
+        @ivar default_value: the value this StateVariable should have by default when created
+                             for another instance of in the service
+        @ivar allowed_values: a C{list} of values this StateVariable can have
+        @ivar has_vendor_values: boolean if there are values outside the allowed_values list too
+        @ivar allowed_value_range: a C{dict} of 'minimum','maximum' and 'step' values
+        @ivar moderated: boolean, True if this StateVariable should only be evented via a LastChange event
+
+        """
+
+        # FIXME
+        # we should raise an Exception when there as a StateVariable with that name already
+
+        if evented == 'never':
+            send_events = 'no'
+        else:
+            send_events = evented
+        new_variable = variable.StateVariable(self,name,implementation,instance,send_events,
+                                                   data_type,allowed_values)
+        if default_value == None:
+            new_variable.default_value = ''
+        else:
+            new_variable.default_value = new_variable.old_value = new_variable.value = default_value
+
+        new_variable.dependant_variable = dependant_variable
+        new_variable.has_vendor_values = has_vendor_values
+        new_variable.allowed_value_range = allowed_value_range
+        new_variable.moderated = moderated
+        if evented == 'never':
+            new_variable.never_evented = True
+        self._variables.get(instance)[name] = new_variable
+        return new_variable
+
+    def register_vendor_action(self,name,implementation,arguments=None,needs_callback=True):
+        """
+        enables a backend to add an own, vendor defined, Action to the service
+
+        @ivar name: the name of the new Action
+        @ivar implementation: either 'optional' or 'required'
+        @ivar arguments: a C{list} if argument C{tuples},
+                         like (name,direction,relatedStateVariable)
+        @ivar needs_callback: this Action needs a method in the backend or service class
+        """
+
+        # FIXME
+        # we should raise an Exception when there as an Action with that name already
+        # we should raise an Exception when there is no related StateVariable for an Argument
+
+        """ check for action in backend """
+        callback = getattr(self.backend, "upnp_%s" % name, None)
+
+        if callback == None:
+            """ check for action in ServiceServer """
+            callback = getattr(self, "upnp_%s" % name, None)
+
+        if( needs_callback == True and
+            callback == None):
+            """ we have one or more 'A_ARG_TYPE_' variables
+                issue a warning for now
+            """
+            if implementation == 'optional':
+                self.info('%s has a missing callback for %s action %s, action disabled' % (self.id,implementation,name))
+                return
+            else:
+                if((hasattr(self,'implementation') and self.implementation == 'required') or
+                    not hasattr(self,'implementation')):
+                    self.warning('%s has a missing callback for %s action %s, service disabled' % (self.id,implementation,name))
+                raise LookupError,"missing callback"
+
+        arguments_list = []
+        for argument in arguments:
+            arguments_list.append(action.Argument(argument[0],argument[1].lower(),argument[2]))
+
+        new_action = action.Action(self, name, implementation, arguments_list)
+        self._actions[name] = new_action
+        if callback != None:
+            new_action.set_callback(callback)
+            self.info('Add callback %s for %s/%s' % (callback, self.id, name))
+        return new_action
+
     def init_var_and_actions(self):
         desc_file = util.sibpath(__file__, os.path.join('xml-service-descriptions', '%s%d.xml' % (self.id, int(self.version))))
         tree = ET.parse(desc_file)
@@ -672,11 +781,16 @@ class ServiceServer(log.Loggable):
             name = action_node.findtext('name')
             implementation = 'required'
             needs_callback = False
+            if action_node.attrib.get('{urn:schemas-beebits-net:service-1-0}X_needs_backend',
+                                        None) != None:
+                needs_callback = True
             if action_node.find('Optional') != None:
                 implementation = 'optional'
-                if action_node.find('Optional').attrib.get(
+                if(action_node.find('Optional').attrib.get(
                                         '{urn:schemas-beebits-net:service-1-0}X_needs_backend',
-                                        None) != None:
+                                        None) != None or
+                    action_node.attrib.get('{urn:schemas-beebits-net:service-1-0}X_needs_backend',
+                                            None) != None):
                     needs_callback = True
 
             arguments = []
@@ -735,6 +849,14 @@ class ServiceServer(log.Loggable):
             implementation = 'required'
             if action_node.find('Optional') != None:
                 implementation = 'optional'
+
+            #if implementation == 'optional':
+            #    for action_object in self._actions.values():
+            #        if name in [a.get_state_variable() for a in action_object.arguments_list]:
+            #            break
+            #    else:
+            #        continue
+
             send_events = var_node.findtext('sendEventsAttribute')
             data_type = var_node.findtext('dataType')
             values = []
@@ -744,6 +866,10 @@ class ServiceServer(log.Loggable):
                                                            implementation,
                                                            instance, send_events,
                                                            data_type, values)
+
+            dependant_variable = var_node.findtext('{urn:schemas-beebits-net:service-1-0}X_dependantVariable')
+            if dependant_variable:
+                self._variables.get(instance)[name].dependant_variable = dependant_variable
             default_value = var_node.findtext('defaultValue')
             if default_value:
                 self._variables.get(instance)[name].set_default_value(default_value)
@@ -753,9 +879,7 @@ class ServiceServer(log.Loggable):
                                         None)
                 if never_evented is not None:
                     self._variables.get(instance)[name].set_never_evented(never_evented)
-            dependant_variable = var_node.findtext('{urn:schemas-beebits-net:service-1-0}X_dependantVariable')
-            if dependant_variable:
-                self._variables.get(instance)[name].dependant_variable = dependant_variable
+
             allowed_value_list = var_node.find('allowedValueList')
             if allowed_value_list != None:
                 vendor_values = allowed_value_list.attrib.get(
@@ -792,6 +916,11 @@ class ServiceServer(log.Loggable):
                 self._variables.get(instance)[name].set_allowed_value_range(**range)
                 if vendor_values != None:
                     self._variables.get(instance)[name].has_vendor_values = True
+            elif service_range_defaults:
+                variable_range_defaults = service_range_defaults.get(name)
+                if variable_range_defaults != None:
+                    self._variables.get(instance)[name].set_allowed_value_range(**variable_range_defaults)
+                    self._variables.get(instance)[name].has_vendor_values = True
 
         for v in self._variables.get(0).values():
             if isinstance( v.dependant_variable, str):
@@ -799,8 +928,17 @@ class ServiceServer(log.Loggable):
 
 class scpdXML(static.Data):
 
-    def __init__(self, server, control):
+    def __init__(self, server, control=None):
+        self.service_server = server
+        self.control = control
+        static.Data.__init__(self, None, 'text/xml')
 
+    def render(self, request):
+        if self.data == None:
+            self.data = self.build_xml()
+        return static.Data.render(self,request)
+
+    def build_xml(self):
         root = ET.Element('scpd')
         root.attrib['xmlns']='urn:schemas-upnp-org:service-1-0'
         e = ET.SubElement(root, 'specVersion')
@@ -808,7 +946,7 @@ class scpdXML(static.Data):
         ET.SubElement( e, 'minor').text = '0'
 
         e = ET.SubElement( root, 'actionList')
-        for action in server._actions.values():
+        for action in self.service_server._actions.values():
             s = ET.SubElement( e, 'action')
             ET.SubElement( s, 'name').text = action.get_name()
             al = ET.SubElement( s, 'argumentList')
@@ -819,7 +957,7 @@ class scpdXML(static.Data):
                 ET.SubElement( a, 'relatedStateVariable').text = argument.get_state_variable()
 
         e = ET.SubElement( root, 'serviceStateTable')
-        for var in server._variables[0].values():
+        for var in self.service_server._variables[0].values():
             s = ET.SubElement( e, 'stateVariable')
             if var.send_events == True:
                 s.attrib['sendEvents'] = 'yes'
@@ -845,8 +983,7 @@ class scpdXML(static.Data):
                          if value != None:
                             ET.SubElement( avl, name).text = str(value)
 
-        self.xml = """<?xml version="1.0" encoding="utf-8"?>""" + ET.tostring( root, encoding='utf-8')
-        static.Data.__init__(self, self.xml, 'text/xml')
+        return """<?xml version="1.0" encoding="utf-8"?>""" + ET.tostring( root, encoding='utf-8')
 
 from twisted.python.util import OrderedDict
 
